package backup.client;

import java.net.ConnectException;
import java.net.MalformedURLException;
import java.util.*;
import org.apache.xmlrpc.*;

/**
 * Connection for XML-RPC calls to BackupServer.
 * 
 * @author <a href="mailto:mauset@idi.ntnu.no">Tore Mauset</a>
 */
public class Connection {
    XmlRpcClient server;
    BackupClient bc;

    /**
     * Constructs a new Connection.
     * 
     * @param bc parent BackupClient
     * @param host server address
     * @param port server port
     */
    public Connection(BackupClient bc, String host, int port) {
        this.bc = bc;

        String server_url = "http://" + host + ":" + Integer.toString(port);

        try {
            server = new XmlRpcClient(server_url);
        }
        catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }

    /**
     * Executes a XML-RPC call on the server.
     * 
     * @param action method to execute on the server
     * @param query parameters
     * @return result from server
     */
    public synchronized Hashtable send(String action, Vector query) {
        try {
            // Call the server, and get our result.
            Hashtable result = (Hashtable)server.execute("BackupServer."
                    + action, query);

            // Record next authentification challenge
            String challenge = (String)result.remove("challenge");
            bc.setChallenge(challenge);

            return result;
        }
        catch (ConnectException e) {
            bc.warn("Connection refused.");
        }
        catch (XmlRpcException e) {
            System.err.println("XML-RPC Fault #" + Integer.toString(e.code)
                    + ": " + e.toString());
        }
        catch (Exception e) {
            System.err.println("Exception: " + e.toString());
        }
        return null;
    }

}
