package backup.client;

import java.io.*;

/**
 * Reader for <code>InputStream</code> from rdiff-backup.
 * 
 * @author <a href="mailto:mauset@idi.ntnu.no">Tore Mauset</a>
 */
class StreamReader extends Thread {
    private InputStream is;
    private UserInterface ui;

    /**
     * Constructs a new StreamReader.
     * 
     * @param is InputStream to read from
     * @param ui client's UserInterface
     */
    StreamReader(InputStream is, UserInterface ui) {
        this.is = is;
        this.ui = ui;
    }

    /**
     * Start a separate thread for reading from the InputStream.
     */
    public void run() {
        try {
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            while ((line = br.readLine()) != null) {
                if (line.startsWith("Processing changed file")) {
                    String filename = line.substring(24);
                    if (!filename.equals(".")) {
                        ui.processingFile(filename);
                    }
                }
                else if (line.startsWith("Starting restore of")) {
                    // TODO Parse line and tell UI about it
                    System.out.println(line);
                }
                else if (line.endsWith("specify --force to overwrite.")) {
                    String filename = line.substring(28, line.length() - 46);
                    ui
                            .warn("\""
                                    + filename
                                    + "\" already exists, specify force option to overwrite.\n"
                                    + "WARNING! If target is a directory its current contents "
                                    + "will be DELETED before restore starts!");
                }
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }

    }
}

/**
 * * Wrapper for executing rdiff-backup.
 * 
 * @author <a href="mailto:mauset@idi.ntnu.no">Tore Mauset</a>
 */
public class RdiffWrapper {
    private Runtime rt;
    private UserInterface ui;

    /**
     * Constructs a new Rdiffwrapper.
     * 
     * @param ui UserInterface of parent BackupClient
     */
    public RdiffWrapper(UserInterface ui) {
        this.ui = ui;
    }

    /**
     * Executes rdiff-backup.
     * 
     * @param cmd complete command to execute
     */
    public Boolean start(String[] cmd) {
        rt = Runtime.getRuntime();
        try {
            Process proc = rt.exec(cmd);
            StreamReader errors = new StreamReader(proc.getErrorStream(), ui);
            StreamReader output = new StreamReader(proc.getInputStream(), ui);
            errors.start();
            output.start();
            int exitVal = proc.waitFor();
            if (exitVal != 0) {
                return false;
            }
            else {
                return true;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
            return false;
        }

    }
}
