package backup.client;

import java.util.*;
import java.net.ConnectException;
import java.security.cert.*;
import javax.net.ssl.*;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.secure.*;

/**
 * SSl connection for XML-RPC calls to BackupServer.
 * 
 * @author <a href="mailto:mauset@idi.ntnu.no">Tore Mauset</a>
 *
 */
public class SSLConnection {
    
    SecureXmlRpcClient server;
    BackupClient bc;
   
    /**
     * Constructs a new SSLConnection.
     * 
     * @param bc parent BackupClient
     * @param host server address
     * @param port server port
     */
    public SSLConnection(BackupClient bc, String host, int port) {
        this.bc = bc;
        
        try {
            SSLContext sslContext =  SSLContext.getInstance("SSL");
            sslContext.init(null, new X509TrustManager[] {
                new X509TrustManager() {
                    public void checkClientTrusted(X509Certificate[] chain, String authType) {
                    }
                    public void checkServerTrusted(X509Certificate[] chain, String authType) {
                    }
                    public X509Certificate[] getAcceptedIssuers() {
                        return null;
                    }
                }
            }, null);
            HttpsURLConnection.setDefaultSSLSocketFactory(sslContext.getSocketFactory());
        }
        catch (Exception e) {
            // SSL connection configure error
            e.printStackTrace();            
        }
        
        HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
            public boolean verify(String hostname, SSLSession session) {
                // Ignore FODN not matched with certificate common name
                return true;
            }
        });
        
        try {
            String address = "https://" + host + ":" + port;
            server = new SecureXmlRpcClient(address);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Executes a XML-RPC call on the server.
     * 
     * @param action method to execute on the server
     * @param query parameters
     * @return result from server
     */
    public synchronized Hashtable send(String action, Vector query) {
        try {
            // Call the server, and get our result.
            Hashtable result = (Hashtable)server.execute("BackupServer."
                    + action, query);

            // Record next authentification challenge
            String challenge = (String)result.remove("challenge");
            bc.setChallenge(challenge);

            return result;
        }
        catch (ConnectException e) {
            bc.warn("Connection refused.");
        }
        catch (XmlRpcException e) {
            System.err.println("XML-RPC Fault #" + Integer.toString(e.code)
                    + ": " + e.toString());
        }
        catch (Exception e) {
            System.err.println("Exception: " + e.toString());
        }
        return null;
    }
}
