package backup.server;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import java.text.SimpleDateFormat;

/**
 * Command line interface for <code>BackupServer</code>
 * 
 * @author <a href="mailto:mauset@idi.ntnu.no">Tore Mauset</a>
 */
public class Cli extends Thread {

    private BackupServer bs;
    private boolean running = true;
    private String last = "";

    /**
     * Constructs a new Cli.
     * 
     * @param bs parent <code>BackupServer</code>
     */
    public Cli(BackupServer bs) {
        this.bs = bs;
    }

    /**
     * Start a separate thread for keeping track of keyboard input.
     */
    public void run() {
        try {
            InputStream inputstream = System.in;
            InputStreamReader inputstreamreader = new InputStreamReader(
                    inputstream);
            BufferedReader bufferedreader = new BufferedReader(
                    inputstreamreader);
            String string = null;

            System.out.print("> ");
            last = "prompt";
            while (running) {
                string = bufferedreader.readLine();
                last = "";
                if (!string.equals("")) {
                    in(string);
                }
                if (running) {
                    System.out.print("> ");
                    last = "prompt";
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Prints a status message.
     * 
     * @param msg message
     */
    public void status(String msg) {
        if (last.equals("prompt")) {
            System.out.println("");
            last = "msg";
        }
        Date today = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss");
        String time = formatter.format(today);
        System.out.println(time + " " + msg);
    }

    /**
     * Prints a warning message.
     * 
     * @param msg message
     */
    public void warn(String msg) {
        if (last.equals("prompt")) {
            System.out.println("");
            last = "msg";
        }
        Date today = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss");
        String time = formatter.format(today);
        System.out.println(time + " [!] " + msg);
    }

    /*
     * Processes one line of user input read from the keyboard.
     */
    private void in(String msg) {
        String[] tmp = msg.split(" ", 2);
        String cmd = tmp[0];
        String allargs = "";
        String[] args = {};

        if (tmp.length > 1) {
            allargs = tmp[1];
            args = allargs.split(" ");
        }

        if (cmd.equals("exit") && args.length == 0) {
            bs.exit();
            running = false;
        }
        else if (cmd.equals("list")) {
            listUsers();
        }
        else if (cmd.equals("help")) {
            usage();
        }
    }

    /*
     * Prints help.
     */
    private void usage() {
        status("list     list logged in users");
        status("exit     shut down server");
        status("help     this text");
    }

    /*
     * Prints a list of users currently logged in.
     */
    private void listUsers() {
        status("Current users:");

        ArrayList userlist = bs.getUsers();

        for (int i = 0; i < userlist.size(); i++) {
            status((String)userlist.get(i));
        }
    }

}
